"""
Core functionality for interacting with the HNSearch API.
"""

import requests as req
import simplejson as json

from . import thriftdb
from .times import date_format


class News(object):

    def __init__(self):
        self.url = "http://api.thriftdb.com/api.hnsearch.com/items/_search"

    def get(self, **params):
        """Perform a GET request."""
        # Default to the maximum 100 results
        if "limit" not in params:
            params["limit"] = 100
        params = thriftdb.convert(params)
        data = req.get(self.url, params=params)
        self.request = data
        return json.loads(data.content)

    def date(self, day, **params):
        """Query a specific date."""
        params['day'] = date_format(day)
        return self.get(**params)

    def between(self, start, finish, **params):
        """Filter between specific datetime intervals."""
        start = start.strftime("%Y-%m-%dT%H:%M:%SZ")
        finish = finish.strftime("%Y-%m-%dT%H:%M:%SZ")
        value = "[%s TO %s]" % (start, finish)
        params["filter[fields][create_ts]"] = value
        return self.get(**params)

    def facet(self, term, **params):
        """Facets are almost like searching what can be searched."""
        params['facet'] = term
        return self.get(**params)

    def filter(self, condition, **params):
        """Filter the results to a specific condition."""
        params['filter'] = condition
        return self.get(**params)

    def search(self, term, **params):
        """Perform a search against the HNSearch API."""
        params['q'] = term
        return self.get(**params)



"""
Command line goodness.
"""

import sys
from argparse import ArgumentParser
import simplejson as json

try:
    from pygments import highlight
    from pygments.formatters import Terminal256Formatter as Term256
    from pygments.lexers import JavascriptLexer
    has_pygments = True
except ImportError:
    has_pygments = False

from .core import News
from .times import date_format


def create_parser():
    """Create the Hacker News command line parser."""
    description = 'A simple and easy-to-use CLI for the HNSearch API.'
    parser = ArgumentParser(prog='hn', description=description)
    parser.add_argument('search', metavar='S', type=str, nargs='*',
                        help="A search term to query for.")
    parser.add_argument('-n', '--limit', metavar='N', type=int,
                        help="number of results to return (max 100)")
    parser.add_argument('-s', '--start', metavar='N', type=int,
                        help="ordinal position of first result")
    parser.add_argument('-d', '--day', metavar='YYYY-MM-DD', type=str,
                        help="filter by a specific date")
    parser.add_argument('-t', '--time', type=str, nargs='+',
                        help="filter between specific times")
    parser.add_argument('-S', '--sort', metavar='S', nargs='*',
                        help="list of pairs to sort results")
    parser.add_argument('-T', '--type', metavar='TYPE',
                        help="type of items returned (comment or submission)")
    parser.add_argument('-u', '-U', '--username', metavar='NAME',
                        help="filter for a specific username")
    parser.add_argument('--hits', action='store_true',
                        help="return only the number of results encountered")
    return parser


def search(options):
    """Perform a search against the HNSearch API."""
    params = {}
    if options.search:
        term = ' '.join(options.search)
        params['q'] = term
    if options.limit:
        params['limit'] = options.limit
    if options.start:
        params['start'] = options.start
    if options.day:
        params['day'] = date_format(options.day)
    if options.time:
        params['time'] = options.time
    if options.sort:
        sort = options.sort
        if len(sort) == 1:
            sort.append('desc')
        params['sortby'] = ' '.join(sort)
    if options.type:
        params['type'] = options.type
    if options.username:
        params['username'] = options.username
    data = News().get(**params)
    return present(options, data)


def present(options, data):
    """Determine what to do with the data based on CLI options."""
    if options.hits:
        print data['hits']
    else:
        return decorate(data)


def decorate(stream, indent=2):
    """Turn JSON into something readable."""
    print_formatted_json(sys.stdout, stream, indent)
    sys.stdout.write('\n')


def print_formatted_json(stream, json_data, indentation):
    """Format and print JSON."""
    formatted = json.dumps(json_data, indent=indentation)
    if has_pygments and getattr(stream, 'isatty', lambda: False)():
        formatter = Term256()
        lexer = JavascriptLexer()
        formatted = highlight(formatted, formatter=formatter, lexer=lexer)
    print >> stream, formatted.rstrip()



from .core import News


def get(**params):
    """Perform a GET request against the HNSearch API."""
    return News().get(**params)


def day(date, **params):
    """Search a specific date."""
    return News().date(date, **params)


def date(date, **params):
    """Search a specific date."""
    return News().date(date, **params)


def between(self, start, finish, **params):
    """Search between two specific datetimes."""
    return News().between(start, finish, **params)


def search(term, **params):
    """Query the HNSearch API."""
    return News().search(term, **params)



"""
Install the package and script.
"""

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name="hn",
    version="0.2.1",
    description="A CLI and Python library for using the HNSearch API.",
    keywords="hn, ycombinator, hacker news, newsyc",
    author="Zach Williams",
    author_email="hey@zachwill.com",
    url="https://github.com/zachwill/hn",
    license="MIT",
    packages=[
        "yc"
    ],
    scripts=[
        "hn"
    ],
    install_requires=[
        "pygments",
        "requests",
        "simplejson"
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP',
    ],
    test_suite="test.py",
    tests_require=[
        "mock"
    ]
)



"""
ThriftDB API:  http://www.thriftdb.com/documentation/rest-api/search-api

Turn a dictionary of sane key-value pairs to the somewhat crazy ThriftDB
key-value parameters.
"""

from .times import hour_format


def convert(data):
    """Where the conversion magic happens."""
    if 'day' in data:
        key = 'filter[fields][create_ts]'
        date = data.pop('day')
        if 'time' in data:
            start, end = data['time']
            start = hour_format(start)
            end = hour_format(end)
        else:
            start = '00:00:00'
            end = '23:59:59'
        value = "[%sT%sZ TO %sT%sZ]" % (date, start, date, end)
        data[key] = value
    if 'type' in data:
        # The type can't be plural (i.e. comments, submissions).
        key = 'filter[fields][type]'
        data[key] = data.pop('type').rstrip('s')
    if 'username' in data:
        value = data.pop('username')
        data['filter[fields][username]'] = value
    return data



"""
Abstractions for dealing with dates. I don't know if you knew this, but dealing
with dates and times in Python kind of blows.
"""

import datetime
from collections import Iterable


def date_format(date):
    """Convert the given input to a date."""
    if isinstance(date, str):
        if '-' not in date:
            if len(date) == 8:
                # It's a YYYYMMDD date.
                year = date[:4]
                month = date[4:6]
                day = date[6:]
                date = "%s-%s-%s" % (year, month, day)
            else:
                # Then it's MMDDYY.
                year = date[4:]
                month = date[:2]
                day = date[2:4]
                date = "20%s-%s-%s" % (year, month, day)
        elif len(date) == 8:
            # Then it's in MM-DD-YY format.
            month, day, year = date.split('-')
            date = "20%s-%s-%s" % (year, month, day)
    elif isinstance(date, Iterable):
        date = '-'.join(str(n) for n in date)
    else:
        date = date.strtime("%Y-%m-%d")
    return date


def hour_format(hour):
    """Turn an string into the correct HNSearch API format."""
    hour = hour.lower()
    length = len(hour)
    if length == 1:
        formatted = "0%s:00:00" % hour
    elif length == 2:
        formatted = "%s:00:00" % hour
    else:
        time = hour[-2:]
        hour = hour[:-2]
        if length == 3:
            if time == 'am':
                formatted = "0%s:00:00" % hour
            else:
                hour = str(int(hour) + 12)
                formatted = "%s:00:00" % hour
        else:
            if time == 'am' and hour == '12':
                hour = '00'
            elif time == 'pm' and hour.startswith('0'):
                hour = str(int(hour) + 12)
            formatted = "%s:00:00" % hour
    return formatted
